#!/bin/sh
## build glibc-$VERSION for KISS Linux
## Gilbert Ashley <amigo@ibiblio.org>
## Adapted from an old glibc build-script by Patrick Volkerding

CWD=$(pwd)

VERSION=${VERSION:-2.11.1}
# libidn now included with glibc>=2.11
# LIBIDN_VERSION=2.9
# LIBIDN_SOURCE=glibc-libidn-$LIBIDN_VERSION.tar.gz
CVS_VERSION=2.11.1
# be sure the kernel headers are installed in the right place:
# --with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include
KERNEL_HEADERS=2.6.31.6
MIN_KERNEL=2.6.27
TZDATA=tzdata2009s
TZCODE=tzcode2009r
BUILD=${BUILD:-1}
PACKAGER="Gilbert Ashley <amigo@ibiblio.org>"

ARCH=${ARCH:-i586}

TARGET=$ARCH-kiss-linux
PKG_META=pkg-meta
PKG_FORMAT=txz
MAN_COMPRESS='gzip -9'

# PATCHLIST
# Use old-style locale directories rather than a single /usr/lib/locale/locale-archive file:
  ## glibc.locale.no-archive.diff.gz
# The is_IS locale is causing a strange error about the "echn" command
# not existing.  This patch reverts is_IS to the version shipped in glibc-2.5:
  ## is_IS.diff.gz
# Fix NIS netgroups:
  ## glibc.nis-netgroups.diff.gz
# This patch makes glibc consider only the first three digits of the
# kernel version and drops any trailing non-digit characters:
  ## glibc.kernelversion.diff.gz | patch -p1 --verbose || exit 1
# Support ru_RU.CP1251 locale:
  ## glibc.ru_RU.CP1251.diff.gz | patch -p1 --verbose || exit 1
# Fix missing MAX macro in getcwd.c:
  ## glibc.getcwd.max.macro.diff.gz | patch -p1 --verbose || exit 1
# Fix a dumb warning ( from the crosstool-ng glibc fixes)
  # i386-preferred-stack-boundary.patch.gz

PATCHLIST='glibc.locale.no-archive.diff.gz
glibc.nis-netgroups.diff.gz
glibc.kernelversion.diff.gz
glibc.ru_RU.CP1251.diff.gz
glibc.getcwd.max.macro.diff.gz
i386-preferred-stack-boundary.patch.gz'

# Temporary build location.
TMP=${TMP:-$CWD/Glibc-Project}
if [ ! -d $TMP ]; then
  mkdir -p $TMP
fi

# The version(s) should be carefully selected to go with the glibc version
if [ ! -d /usr/src/linux-${KERNEL_HEADERS}/usr/include ]; then
  echo "Missing:  /usr/src/linux-${KERNEL_HEADERS}"
  echo "Exiting..."
  exit
fi

LIBDIRSUFFIX=
case $ARCH in
	i386)	OPTIMIZE="-O3 -pipe -march=i386 -mcpu=i686" ;;
	i486)	OPTIMIZE="-O3 -pipe -march=i486 -mtune=i686" ;;
	i586)	OPTIMIZE="-O3 -pipe -march=i586 -mtune=i686" ;;
	i686)	OPTIMIZE="-O3 -pipe -march=i686" ;;
	athlon)	OPTIMIZE="-O3 -pipe -march=athlon" ;;
	s390)	OPTIMIZE="-O3 -pipe" ;;
	s390x)	OPTIMIZE="-O3 -pipe" ;;
	x86_64)	OPTIMIZE="-O3 -fPIC -pipe" ; LIBDIRSUFFIX="64" ;;
	armel)	OPTIMIZE="-O3 -pipe -march=armv4t" ;;
	arm)	OPTIMIZE="-O3 -pipe -march=armv4 -mtune=xscale" ;;
	*)		OPTIMIZE="-O3 -pipe" ;;
esac

# This is going to be the initial 'install_root' location
export PKG_TREE=$TMP/package-glibc-incoming-tree
# package-tree for each of the packages
PKG_FULL=$TMP/package-glibc
PKG_SOLIBS=$TMP/package-glibc-solibs
PKG_TIME=$TMP/package-glibc-zoneinfo
PKG_I18N=$TMP/package-glibc-i18n
PKG_DEVEL=$TMP/package-glibc-devel
PKG_DOCS=$TMP/package-glibc-docs

create_dirs() {
# Empty these locations first:
for dir in $PKG_FULL $PKG_SOLIBS $PKG_TIME $PKG_I18N $PKG_DEVEL $PKG_DOCS ; do
  if [ -d $dir ]; then
    rm -rf $dir
  fi
  mkdir -p $dir
done

# Create the PKG_TREE (used as install_root
mkdir -p $PKG_TREE/lib${LIBDIRSUFFIX}
mkdir -p $PKG_TREE/sbin
mkdir -p $PKG_TREE/usr/bin
mkdir -p $PKG_TREE/usr/lib${LIBDIRSUFFIX}
mkdir -p $PKG_TREE/usr/sbin
mkdir -p $PKG_TREE/usr/include
mkdir -p $PKG_TREE/usr/share
mkdir -p $PKG_TREE/usr/share/doc
mkdir -p $PKG_TREE/usr/share/man
mkdir -p $PKG_TREE/usr/share/info
mkdir -p $PKG_TREE/var/db/nscd
mkdir -p $PKG_TREE/var/run/nscd
}

#
apply_patches() {
	cd $TMP/glibc-$CVS_VERSION
	for p in $PATCHLIST ; do
		zcat $CWD/glibc-patches/$p | patch -p1 --verbose || exit 1
	done
	# Update the timezone information.
	( cd timezone
		tar xzf $CWD/timezone/$TZDATA.tar.gz
		chown root:root *
		mv yearistype.sh yearistype
		chmod 644 *
		chmod 755 yearistype
		mkdir tzcode
		cd tzcode
		tar xzf $CWD/timezone/$TZCODE.tar.gz
		chown -R root:root .
		chmod 644 *
		cp -a *.c *.h ..
	)
}

#
write_meta_data() {
	THIS_PACKAGE="$1"
	mkdir -p "$THIS_PACKAGE"/install
	case $THIS_PACKAGE in
		$PKG_FULL) echo "# Package: glibc-$VERSION-$ARCH-$BUILD.$PKG_FORMAT" > "$THIS_PACKAGE"/install/$PKG_META ;;
		$PKG_SOLIBS) echo "# Package: glibc-solibs-$VERSION-$ARCH-$BUILD.$PKG_FORMAT" > "$THIS_PACKAGE"/install/$PKG_META ;;
		$PKG_DEVEL) echo "# Package: glibc-devel-$VERSION-$ARCH-$BUILD.$PKG_FORMAT" > "$THIS_PACKAGE"/install/$PKG_META ;;
	esac
	echo "# Created by: $PACKAGER" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Hostname: $(hostname)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Date: $(date)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Processor: $(uname -i) $(uname -m)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Kernel: $(uname -r) $(uname -v)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Compiler: GCC-$(gcc -dumpversion) $(gcc -dumpmachine)" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Source name: glibc-$CVS_VERSION.tar.bz2" >> "$THIS_PACKAGE"/install/$PKG_META
	if [[ "$PATCHLIST" != "" ]] ; then
		echo "# Patches applied:" >> "$THIS_PACKAGE"/install/$PKG_META
		for PATCH in $PATCHLIST ; do
			echo "	${PATCH##*/}" >> "$THIS_PACKAGE"/install/$PKG_META
		done
	fi
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
	echo "# Build configuration:" >> "$THIS_PACKAGE"/install/$PKG_META
	cat "$CWD"/config-line.txt >> "$THIS_PACKAGE"/install/$PKG_META
	echo "" >> "$THIS_PACKAGE"/install/$PKG_META
}

# Correct library paths in doinst.sh for x86_64.
install_doinst() {
	sed -e "s#SFX#$LIBDIRSUFFIX#g" $CWD/doinst.sh.in > install/doinst.sh
	sed -i "s#VERSION#$VERSION#g" install/doinst.sh
	if [ "$ARCH" = "x86_64" ] ; then
		sed -i "s#ld-linux.so.2#ld-linux-x86-64.so.2#g" install/doinst.sh
	fi
}

install_desc() {
	DESC_FILE=$1
	sed -e "s#PACKAGER#$PACKAGER#g" $DESC_FILE > install/pkg-desc
}

# Begin extract/compile:
unpack_sources() {
	cd $TMP
	rm -rf glibc-$CVS_VERSION
	tar xvf $CWD/glibc-$CVS_VERSION.tar.xz
	cd glibc-$CVS_VERSION
	# libidn is now included with glibc>=2.11
	#tar xf $CWD/$LIBIDN_SOURCE
	#mv glibc-libidn-$LIBIDN_VERSION libidn || exit 1
}

# correct perms and ownership
fix_source_perms() {
	cd $TMP/glibc-$CVS_VERSION
	chown -R root:root .
	find . -perm 666 -exec chmod 644 {} \;
	find . -perm 664 -exec chmod 644 {} \;
	find . -perm 600 -exec chmod 644 {} \;
	find . -perm 444 -exec chmod 644 {} \;
	find . -perm 400 -exec chmod 644 {} \;
	find . -perm 440 -exec chmod 644 {} \;
	find . -perm 777 -exec chmod 755 {} \;
	find . -perm 775 -exec chmod 755 {} \;
	find . -perm 511 -exec chmod 755 {} \;
	find . -perm 711 -exec chmod 755 {} \;
	find . -perm 555 -exec chmod 755 {} \;
	# Clean up any CVS cruft
	find . -type d -name CVS -exec rm -r {} \; 2> /dev/null
}

build_sources() {
	cd $TMP/glibc-$CVS_VERSION
	# Make build directory:
	mkdir build-glibc-$VERSION
	cd build-glibc-$VERSION

	echo "CFLAGS=\"-g $OPTIMIZE\" \\
	../configure  --prefix=/usr --enable-kernel=${MIN_KERNEL} --with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include \\
	--enable-add-ons=libidn,nptl --disable-profile --infodir=/usr/share/info --mandir=/usr/share/man --with-tls \\
	--with-__thread --without-cvs $TARGET" > ${CWD}/config-line.txt

	echo "Building GLIBC"
	CFLAGS="-g $OPTIMIZE" \
	../configure \
		--prefix=/usr \
		--libdir=/usr/lib${LIBDIRSUFFIX} \
		--enable-kernel=${MIN_KERNEL} \
		--with-headers=/usr/src/linux-${KERNEL_HEADERS}/usr/include \
		--enable-add-ons=libidn,nptl \
		--disable-profile \
		--infodir=/usr/share/info \
		--mandir=/usr/share/man \
		--with-tls \
		--with-__thread \
		--without-cvs \
		$TARGET
	
	make -j4 || exit 1
}

install_pkg_tree() {
	if [ -d $PKG_TREE ] ; then
	echo "Removing stale package-tree content"
	rm -rf $PKG_TREE
	fi
	echo "Installing GLIBC in:"
	echo "$PKG_TREE"
	mkdir -p $PKG_TREE
	cd $TMP/glibc-$CVS_VERSION/build-glibc-$VERSION
	make install install_root=$PKG_TREE || exit 1
	make localedata/install-locales install_root=$PKG_TREE || exit 1
	# Remove links from package tree -they are all handled by the doinst.sh scripts
	( cd $PKG_TREE
		find . -type l -exec rm {} \;
	)
}

strip_elf() {
	cd $PKG_TREE
	# Strip ELF objects. Don't use '--strip-unneeded' or you'll be sorry
	strip -g usr/lib${LIBDIRSUFFIX}/lib*.a
	find . | xargs file | grep "executable" | grep ELF | cut -f 1 -d : | xargs strip -g 2> /dev/null
	find . | xargs file | grep "shared object" | grep ELF | cut -f 1 -d : | xargs strip -g 2> /dev/null
	# it's okay to strip binary executables harder
	( cd sbin ; strip * 2> /dev/null )
	( cd usr/bin ; strip * 2> /dev/null )
	( cd usr/sbin ; strip * 2> /dev/null )
	# and the gconv libs also
	( cd usr/lib/gconv ; strip * 2> /dev/null )
}

install_misc() {
	# Back to the sources dir to add some files/docs:
	cd $TMP/glibc-$CVS_VERSION
	# Install the config file for the Name Server Cache Daemon.
	mkdir -p $PKG_TREE/etc
	cat nscd/nscd.conf > $PKG_TREE/etc/nscd.conf.new
	# Install some scripts to help select a timezone:
	mkdir -p $PKG_TREE/var/lib/pkgtool/setup
	cp -a $CWD/timezone/setup.timeconfig $PKG_TREE/var/lib/pkgtool/setup
	chown root:root $PKG_TREE/var/lib/pkgtool/setup/setup.timeconfig
	chmod 755 $PKG_TREE/var/lib/pkgtool/setup/setup.timeconfig
	mkdir -p $PKG_TREE/usr/sbin
	cp -a $CWD/timezone/timeconfig $PKG_TREE/usr/sbin
	chown root:root $PKG_TREE/usr/sbin/timeconfig
	chmod 755 $PKG_TREE/usr/sbin/timeconfig
	## Install docs:
	( mkdir -p $PKG_TREE/usr/share/doc/glibc-$VERSION
		cp -a \
		BUGS CONFORMANCE COPYING COPYING.LIB FAQ INSTALL LICENSES NAMESPACE \
		NEWS NOTES PROJECTS README README.libm \
		$PKG_TREE/usr/share/doc/glibc-$VERSION
	)
	# copy this script into the docs, as well
	install -m0644 ${CWD}/${0##*/} $PKG_TREE/usr/share/doc/glibc-$VERSION
	# Fix info dir:
	rm $PKG_TREE/usr/share/info/dir
	${MAN_COMPRESS} $PKG_TREE/usr/share/info/*
	
	# This is junk
	rm $PKG_TREE/etc/ld.so.cache
	( cd $PKG_TREE
		find . -name "*.orig" -exec rm {} \;
	)
}
# Finished fixing up content in the main package tree
####


##############
# here's where the work starts
cd $TMP
create_dirs

#####
skip() {
unpack_sources
fix_source_perms
# Apply patches; exit if any fail.
apply_patches
if [ ! $? = 0 ]; then
  exit 1
fi
build_sources
}
#####

# install and fixup the package-tree content
install_pkg_tree
strip_elf
install_misc

####################
# Now, create the packages

# Create glibc-zoneinfo package
( cd $PKG_TIME
	# Install timezone setup scripts
	mkdir -p var/lib/pkgtool/setup
	cp -a $CWD/timezone/setup.timeconfig var/lib/pkgtool/setup
	chown root:root var/lib/pkgtool/setup/setup.timeconfig
	chmod 755 var/lib/pkgtool/setup/setup.timeconfig
	mkdir -p usr/sbin
	cp -a $CWD/timezone/timeconfig usr/sbin
	chown root:root usr/sbin/timeconfig
	chmod 755 usr/sbin/timeconfig
	# move the timezone info
	mkdir -p usr/share
	( cd usr/share
	  mv $PKG_TREE/usr/share/zoneinfo .
	)
	# install the timezone tools
	mkdir -p usr/bin
	mv $PKG_TREE/usr/bin/tzselect usr/bin
	mkdir -p usr/sbin
	mv $PKG_TREE/usr/sbin/zdump $PKG_TREE/usr/sbin/zic usr/bin
	# install tzcode man-pages
	mkdir -p usr/share/man/man8
	( cd $TMP/glibc-$CVS_VERSION/timezone/tzcode
		cp tzselect.8 zdump.8 zic.8 $PKG_TIME/usr/share/man/man8
	)
	( cd usr/share/man
		find -type f -exec ${MAN_COMPRESS} {} \;
	)
	# installation items
	mkdir install
	cat $CWD/doinst.sh-glibc-zoneinfo > install/doinst.sh
	install_desc $CWD/pkg-desc/pkg-desc.glibc-zoneinfo
	# Create the package:
	rm -f $TMP/glibc-zoneinfo-$VERSION-noarch-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-zoneinfo-$VERSION-noarch-$BUILD.$PKG_FORMAT
)
# end creation of glibc-zoneinfo package

# /usr/bin gencat iconv locale localedef /usr/sbin/iconvconfig
# iconv and iconvconfig go with gconv files
# Create the glibc-i18n package
( cd $PKG_I18N
# these are the bins needed for locales (500K)
	mkdir -p usr/bin
	(cd $PKG_TREE/usr/bin
		mv gencat iconv locale localedef $PKG_I18N/usr/bin
	)
	mkdir -p usr/sbin
	mv $PKG_TREE/usr/sbin/iconvconfig usr/sbin
	
	# move the locale data
	mkdir -p usr/lib${LIBDIRSUFFIX}
	(cd usr/lib${LIBDIRSUFFIX}
		mv $PKG_TREE/usr/lib${LIBDIRSUFFIX}/locale .
	)
	mkdir -p usr/share
	(cd usr/share
		mv $PKG_TREE/usr/share/i18n .
		mv $PKG_TREE/usr/share/locale .
	)
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc-i18n
	# Create the package:
	rm -f $TMP/glibc-i18n-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-i18n-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
)
# end creation of glibc-i18n package

# Create the glibc-devel package
( cd $PKG_DEVEL
	# move the static libs
	mkdir -p usr/lib
	( cd usr/lib
		mv $PKG_TREE/usr/lib/*.a .
	)
	# and the headers
	mkdir -p usr/include
	( cd usr/include
		mv $PKG_TREE/usr/include/* .
	)
	rmdir $PKG_TREE/usr/include
	# info files
	mkdir -p usr/share/info
	( cd usr/share/info
		mv $PKG_TREE/usr/share/info/* .
	)
	rm -rf $PKG_TREE/usr/share/info
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc-devel
	# write the build info
	write_meta_data $PKG_DEVEL
	# Build the package:
	rm -f $TMP/glibc-devel-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-devel-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
)
# end creation of glibc-devel package

# Create the full glibc package
( cd $PKG_FULL
	mv $PKG_TREE/* .
	mkdir -p etc/profile.d
	cp -a $CWD/profile.d/* etc/profile.d
	chown -R root:root etc
	chmod 755 etc/profile.d/*
	mkdir -p var/db/nscd var/run/nscd
	# put the libs in a bogus path for indirect handling
	# by the doinst.sh during installation
	( cd lib${LIBDIRSUFFIX}
		mkdir incoming
		mv *so* incoming
		mv incoming/libSegFault.so .
	)
	
	# installation items
	mkdir install
	install_desc $CWD/pkg-desc/pkg-desc.glibc
	install_doinst $CWD/doinst.sh-glibc
	# write the build info
	write_meta_data $PKG_FULL
	# Build the package:
	rm -f $TMP/glibc-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
	makepkg -l y -c n $TMP/glibc-$VERSION-$ARCH-$BUILD.$PKG_FORMAT
)
# end creation of glibc package

echo
echo "Finished building glibc packages in:"
echo "$TMP"
